package com.example.isUnderPrototype.impls;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.Expression;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import org.eclipse.persistence.expressions.ExpressionBuilder;
import org.eclipse.persistence.internal.jpa.querydef.CompoundExpressionImpl;
import org.eclipse.persistence.internal.jpa.querydef.CriteriaBuilderImpl;
import org.eclipse.persistence.internal.jpa.querydef.FunctionExpressionImpl;
import org.eclipse.persistence.internal.jpa.querydef.InternalSelection;
import org.eclipse.persistence.jpa.JpaCriteriaBuilder;
import org.eclipse.persistence.queries.ReportQuery;

import com.example.isUnderPrototype.IsUnderClause;
import com.example.isUnderPrototype.entity.HierarchyEO;
import com.example.isUnderPrototype.entity.ItemEO;
import com.example.isUnderPrototype.tests.BaseTestCase;

/**
 * <p>This approach uses {@link ReportQuery ReportQueries}
 * to build a subquery <i>and</i> the IN clause predicate in
 * the Eclipselink Expression Language (ELUG). That expression
 * is then casted back (as a {@link FunctionExpressionImpl})
 * into an {@link Expression Expression&lt;Boolean&gt;}. It then
 * uses the {@link CriteriaBuilder#conjunction() conjunction()} with
 * {@link CriteriaBuilder#and(Expression, Expression) and()} to convert
 * it to a Predicate for use with other JPA Predicate chains.</p>
 *
 * <p>Unfortunately, this approach needs to use internals
 * (the {@link #asPredicate(Expression)} method) in order to work.
 * This is because {@link CriteriaBuilderImpl#and(Expression, Expression)}
 * seems to violate the Liskov Substitution Principle. It rejects
 * {@link Expression Expression&lt;Boolean&gt;} that aren't of the
 * CompoundExpressionImpl subtype. So this approach has to build
 * CompoundExpressionImpls to use.</p>
 */
public class UsingInternalLibrary extends BaseIsUnderClauseBuilder {

    @Override
    public Predicate isUnder(Root<ItemEO> root, CriteriaBuilder cb, IsUnderClause data) {
        JpaCriteriaBuilder jcb = (JpaCriteriaBuilder) cb;
        org.eclipse.persistence.expressions.Expression eb = new ExpressionBuilder();
        ReportQuery rq = new ReportQuery(HierarchyEO.class, new ExpressionBuilder());
        rq.setSQLString("SELECT HIERARCHY_ID FROM HIERARCHY START WITH HIERARCHY_ID = " + data.getIsUnder() + " CONNECT BY PRIOR HIERARCHY_ID = PARENT_ID");
        eb = eb.subQuery(rq);
        Expression<Long> elExp = pathToHierarchyId(root, data);
        org.eclipse.persistence.expressions.Expression path = jcb.toExpression(elExp);
        path = path.in(rq);

        return cb.and(cb.conjunction(), asPredicate(jcb.fromExpression(path, Boolean.class)));
    }

    public Expression<Boolean> asPredicate(Expression<Boolean> e) {
        List list = new ArrayList();
        EntityManager em = BaseTestCase.getEM();
        return new CompoundExpressionImpl(em.getMetamodel(), ((InternalSelection) e).getCurrentNode(), list);
    }
}