package com.example.isUnderPrototype.impls;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.Expression;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import com.example.isUnderPrototype.IsUnderClause;
import com.example.isUnderPrototype.entity.ItemEO;

/**
 * <p>This approach uses a function "IS_UNDER", which takes
 * a parent (ancestor) id, a target id, and a type (for using in other
 * hierarchical tables). The function then executes the hierarchical query.</p>
 *
 * <p>This approach works natively with the JPA Criteria API, which is
 * a major plus. However, this approach seems quite slow. The query that is
 * run inside of the PL/SQL function could probably be enhanced, but I think
 * it misses out on some of Oracle DB's query optimization. I think that is
 * the main reason for the slowdown.</p>
 */
public class UsingIsUnderFunction extends BaseIsUnderClauseBuilder {

    @Override
    public Predicate isUnder(Root<ItemEO> root, CriteriaBuilder cb, IsUnderClause data) {
        Expression<Long> fn = cb.function("IS_UNDER", Long.class,
                cb.literal(data.getIsUnder()),
                pathToHierarchyId(root, data),
                cb.literal("HIERARCHY"));
        return cb.equal(fn, cb.literal(Long.valueOf(1L)));
    }
}