package com.example.isUnderPrototype.impls;

import java.util.ArrayList;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaBuilder.In;
import javax.persistence.criteria.Expression;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import org.eclipse.persistence.expressions.ExpressionBuilder;
import org.eclipse.persistence.internal.jpa.querydef.FunctionExpressionImpl;
import org.eclipse.persistence.internal.jpa.querydef.InImpl;
import org.eclipse.persistence.internal.jpa.querydef.SubQueryImpl;
import org.eclipse.persistence.jpa.JpaCriteriaBuilder;

import com.example.isUnderPrototype.IsUnderClause;
import com.example.isUnderPrototype.entity.ItemEO;

/**
 * <p>This approach uses a direct SQL clause.</p>
 *
 * <p>Like {@link UsingIdsUnderFunction}, this approach uses
 * a {@link FunctionExpressionImpl} as an argument to the
 * {@link In#value(Expression)} method. This is currently
 * unsupported, but is patched in this project. The patch
 * has to be turned on with the "usePatch" System property.
 * The patch simply checks before casting to a {@link SubQueryImpl}
 * in {@link InImpl#value(Expression) InImpl.value()}.</p>
 *
 * <p>This approach is our ideal approach because it uses the connect
 * by clause directly in the built query. This allows optimization to
 * occur naturally.</p>
 *
 * <p>Somthing to note is the usage of the
 * {@link org.eclipse.persistence.expressions.Expression#sql(String, java.util.List) sql()}
 * method might be wrong. The base expression in the expression builder
 * has to be the parameter value for this to work. I'm not entirely sure
 * if that is the correct way to use the ExpressionBuilder, but it seems to work.</p>
 */
public class UsingPatch extends BaseIsUnderClauseBuilder {

    @Override
    public Predicate isUnder(Root<ItemEO> root, CriteriaBuilder cb, IsUnderClause data) {
        JpaCriteriaBuilder jcb = (JpaCriteriaBuilder) cb;
        org.eclipse.persistence.expressions.Expression eb = new ExpressionBuilder();
        eb = eb.value(data.getIsUnder());
        org.eclipse.persistence.expressions.Expression processIsUnderFilter = eb
                .sql("SELECT HIERARCHY_ID FROM HIERARCHY START WITH HIERARCHY_ID = ? CONNECT BY PRIOR HIERARCHY_ID = PARENT_ID", new ArrayList<>(0));
        Expression<Long> elExp = pathToHierarchyId(root, data);
        Expression<Long> values = jcb.fromExpression(processIsUnderFilter, Long.class);
        return jcb.in(elExp).value(values);
    }
}