package com.example.isUnderPrototype.impls;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Path;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;

import com.example.isUnderPrototype.IsUnderClause;
import com.example.isUnderPrototype.entity.HierarchyViewEO;
import com.example.isUnderPrototype.entity.ItemEO;

/**
 * <p>This approach uses a view that executes a Cartesian product
 * over the hierarchical table. The view ends up giving all
 * ancestor-descendant relationships just as a closure table would.</p>
 *
 * <p>Like {@link UsingIsUnderFunction}, this approach has the benefit
 * of using the native JPA Criteria API. However, because it executes
 * a Cartesian product, it is the slowest of all solutions. We may be
 * able to tune the optimizer for it, but haven't done so yet.</p>
 *
 */
public class UsingView extends BaseIsUnderClauseBuilder {

    @Override
    public Predicate isUnder(Root<ItemEO> root, CriteriaBuilder cb, IsUnderClause data) {
        CriteriaQuery<HierarchyViewEO> cq = cb.createQuery(HierarchyViewEO.class);
        Root<HierarchyViewEO> rt = cq.from(HierarchyViewEO.class);
        Subquery<Long> sq = cq.subquery(Long.class);
        Path<Long> path = rt.get("id").get("descendent");
        sq.select(path);
        sq.where(cb.equal(rt.get("id").get("ancestor"), cb.literal(data.getIsUnder())));
        return cb.in(pathToHierarchyId(root, data)).value(sq);
    }
}