package com.example.isUnderPrototype.tests;

import java.util.Arrays;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import com.example.isUnderPrototype.IsUnderClause;
import com.example.isUnderPrototype.IsUnderClauseBuilder;
import com.example.isUnderPrototype.entity.HierarchyEO;
import com.example.isUnderPrototype.entity.ItemEO;

public abstract class BaseTestCase {

    protected List<ItemEO> runQueryFor(EntityManager em, long id) {
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<ItemEO> cq = cb.createQuery(ItemEO.class);
        Root<ItemEO> rt = cq.from(ItemEO.class);
        cq = cq.select(rt);
        Predicate p = rt.get("name").isNotNull();
        IsUnderClause data = new IsUnderClause(Arrays.asList("hierarchy", "id"), Long.valueOf(id));
        p = cb.and(p, getImpl().isUnder(rt, cb, data));
        cq = cq.where(p);
        TypedQuery<ItemEO> q = em.createQuery(cq);
        List<ItemEO> eos = q.getResultList();
        return eos;
    }

    protected abstract IsUnderClauseBuilder getImpl();

    public static EntityManager getEM() {
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("sample");
        EntityManager em = emf.createEntityManager();
        return em;
    }

    private static String randomString() {
        return String.valueOf(Math.random());
    }

    public static void cleanData(EntityManager em) {
        try {
            em.getTransaction().begin();
            Query q = em.createNativeQuery("DELETE FROM ITEM");
            q.executeUpdate();
            q = em.createNativeQuery("DELETE FROM HIERARCHY");
            q.executeUpdate();
            em.getTransaction().commit();
        } catch (Exception e) {
            em.getTransaction().rollback();
            e.printStackTrace();
        }
        em.getEntityManagerFactory().getCache().evictAll();
    }

    protected static HierarchyEO makeHierarchyItem(EntityManager em, HierarchyEO parent, long id) {
        HierarchyEO c = new HierarchyEO();
        c.setName(randomString());
        c.setId(Long.valueOf(id));
        c.setParent(parent);
        em.persist(c);
        return c;
    }

    protected static ItemEO makeItem(EntityManager em, HierarchyEO c, long id) {
        ItemEO item = new ItemEO();
        item.setHierarchy(c);
        item.setName(randomString());
        item.setItemId(Long.valueOf(id));
        em.persist(item);
        return item;
    }
}
