package com.example.isUnderPrototype.tests.impl;

import java.util.List;

import javax.persistence.EntityManager;

import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import com.example.isUnderPrototype.entity.HierarchyEO;
import com.example.isUnderPrototype.entity.ItemEO;
import com.example.isUnderPrototype.tests.BaseTestCase;

public abstract class BaseImplTest extends BaseTestCase {

    @BeforeClass
    public static void init() {
        System.setProperty("org.eclipse.persistence.querymonitor", "true");
        EntityManager em = getEM();
        try {
            cleanData(em);
            populateData(em);
        } catch (Exception e) {
            em.close();
            throw e;
        }
    }

    private static void populateData(EntityManager em) {
        try {
            em.getTransaction().begin();
            HierarchyEO root = makeHierarchyItem(em, null, 1L);
            HierarchyEO c1 = makeHierarchyItem(em, root, 2L);
            HierarchyEO c2 = makeHierarchyItem(em, root, 3L);
            HierarchyEO c3 = makeHierarchyItem(em, c1, 4L);

            makeItem(em, root, 1L);
            makeItem(em, root, 2L);

            makeItem(em, c1, 3L);
            makeItem(em, c1, 4L);

            makeItem(em, c2, 5L);
            makeItem(em, c2, 6L);

            makeItem(em, c3, 7L);
            makeItem(em, c3, 8L);

            em.getTransaction().commit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Test
    public void loadsAllUnderTheRoot() {
        EntityManager em = getEM();

        List<ItemEO> eos = runQueryFor(em, 1L);
        Assert.assertNotNull(eos);
        Assert.assertEquals(8, eos.size());
    }

    @Test
    public void loadsItemsUnderMiddleNode() {
        EntityManager em = getEM();

        List<ItemEO> eos = runQueryFor(em, 2L);
        Assert.assertNotNull(eos);
        Assert.assertEquals(4, eos.size());
    }

    @Test
    public void loadsLeafItemsUnderId() {
        EntityManager em = getEM();

        List<ItemEO> eos = runQueryFor(em, 3L);
        Assert.assertNotNull(eos);
        Assert.assertEquals(2, eos.size());

        eos = runQueryFor(em, 4L);
        Assert.assertNotNull(eos);
        Assert.assertEquals(2, eos.size());
    }
}
