package com.example.isUnderPrototype.tests.perf;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.TimeUnit;

import javax.persistence.EntityManager;
import javax.persistence.FlushModeType;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

import com.example.isUnderPrototype.IsUnderClause;
import com.example.isUnderPrototype.IsUnderClauseBuilder;
import com.example.isUnderPrototype.entity.HierarchyEO;
import com.example.isUnderPrototype.entity.ItemEO;
import com.example.isUnderPrototype.impls.UsingIdsUnderFunction;
import com.example.isUnderPrototype.impls.UsingIsUnderFunction;
import com.example.isUnderPrototype.impls.UsingInternalLibrary;
import com.example.isUnderPrototype.impls.UsingPatch;
import com.example.isUnderPrototype.impls.UsingView;
import com.example.isUnderPrototype.tests.BaseTestCase;
import com.google.common.base.Stopwatch;

@RunWith(value = Parameterized.class)
@Ignore
public class PerfTestCase extends BaseTestCase {

    private final int numNodes;
    private final int width;

    public PerfTestCase(int count, int width) {
        this.numNodes = (int) Math.pow(2.0, count * 1.0);
        this.width = width;
    }

    //@formatter:off
    @SuppressWarnings("boxing")
    private static final List<Integer[]> CASES = Arrays.asList(new Integer[][] {
        {1,  1}, {1,  2}, {1,  4}, {1,  8}, {1,  16},
        {2,  1}, {2,  2}, {2,  4}, {2,  8}, {2,  16},
        {3,  1}, {3,  2}, {3,  4}, {3,  8}, {3,  16},
        {4,  1}, {4,  2}, {4,  4}, {4,  8}, {4,  16},
        {5,  1}, {5,  2}, {5,  4}, {5,  8}, {5,  16},
        {6,  1}, {6,  2}, {6,  4}, {6,  8}, {6,  16},
        {7,  1}, {7,  2}, {7,  4}, {7,  8}, {7,  16},
        {8,  1}, {8,  2}, {8,  4}, {8,  8}, {8,  16},
        {9,  1}, {9,  2}, {9,  4}, {9,  8}, {9,  16},
        {10, 1}, {10, 2}, {10, 4}, {10, 8}, {10, 16},
        {11, 1}, {11, 2}, {11, 4}, {11, 8}, {11, 16},
        {12, 1}, {12, 2}, {12, 4}, {12, 8}, {12, 16},
        {13, 1}, {13, 2}, {13, 4}, {13, 8}, {13, 16},
        {14, 1}, {14, 2}, {14, 4}, {14, 8}, {14, 16},
        {15, 1}, {15, 2}, {15, 4}, {15, 8}, {15, 16},
        {16, 1}, {16, 2}, {16, 4}, {16, 8}, {16, 16},
        {17, 1}, {17, 2}, {17, 4}, {17, 8}, {17, 16},
        {18, 1}, {18, 2}, {18, 4}, {18, 8}, {18, 16},
        {19, 1}, {19, 2}, {19, 4}, {19, 8}, {19, 16},
        {20, 1}, {20, 2}, {20, 4}, {20, 8}, {20, 16},
    });
    //@formatter:on

    @Parameters(name = "Case {index}: 2^{0} nodes, {1} width")
    public static Iterable<Integer[]> data() {
        return CASES;
    }

    @Before
    public void setup() {
        EntityManager em = getEM();
        try {
            cleanData(em);
            resetNextHierarchyId();
            resetNextItemId();
            populateData(em);
        } catch (Exception e) {
            em.close();
            throw e;
        }
    }

    private void populateData(EntityManager em) {
        em.setFlushMode(FlushModeType.AUTO);
        try {
            em.getTransaction().begin();
            HierarchyEO root = makeHierarchyItem(em, null, 0L);
            makeNodesUnder(root, 0, em);
            em.getTransaction().commit();
        } catch (Exception e) {
            em.getTransaction().rollback();
            e.printStackTrace();
        }
    }

    private void makeNodesUnder(HierarchyEO root, int i, EntityManager em) {
        if (nodeCountReached()) {
            return;
        }
        List<HierarchyEO> nodesOnThisLevel = new ArrayList<>(width);
        for (int j = 0; j < width; ++j) {
            if (nodeCountReached()) {
                continue;
            }
            nodesOnThisLevel.add(makeNode(root, em));
        }
        for (HierarchyEO node : nodesOnThisLevel) {
            makeNodesUnder(node, i + 1, em);
        }
    }

    private static HierarchyEO makeNode(HierarchyEO parent, EntityManager em) {
        HierarchyEO node = makeHierarchyItem(em, parent, getNextHierarchyId());
        makeItem(em, node, getNextItemId());
        return node;
    }

    private boolean nodeCountReached() {
        return nextHierarchyId > numNodes;
    }

    private static long nextItemId = 1;

    private static synchronized long getNextItemId() {
        return ++nextItemId;
    }

    private static void resetNextItemId() {
        nextItemId = 1;
    }

    private static long nextHierarchyId = 1;

    private static synchronized long getNextHierarchyId() {
        return ++nextHierarchyId;
    }

    private static void resetNextHierarchyId() {
        nextHierarchyId = 1;
    }

    @Override
    protected List<ItemEO> runQueryFor(EntityManager em, long id) {
        CriteriaBuilder cb = em.getCriteriaBuilder();
        CriteriaQuery<ItemEO> cq = cb.createQuery(ItemEO.class);
        Root<ItemEO> rt = cq.from(ItemEO.class);
        cq = cq.select(rt);
        Predicate p = rt.get("name").isNotNull();
        IsUnderClause data = new IsUnderClause(Arrays.asList("hierarchy", "id"), Long.valueOf(id));
        p = cb.and(p, getImpl().isUnder(rt, cb, data));
        cq = cq.where(p);
        TypedQuery<ItemEO> q = em.createQuery(cq);
        q.setMaxResults(100);
        List<ItemEO> eos = q.getResultList();
        return eos;
    }

    @Test
    public void perfIsUnderFunction() {
        Stopwatch sw = Stopwatch.createUnstarted();

        System.setProperty("usePatch", "true");
        this.impl = new UsingIdsUnderFunction();
        sw.start();
        runQueryFor(getEM(), 0L);
        System.setProperty("usePatch", "false");
        System.out.println(String.format("%d %d %d IDS_UNDER", Long.valueOf(numNodes), Long.valueOf(width), Long.valueOf(sw.elapsed(TimeUnit.MICROSECONDS))));
        sw.reset();

        this.impl = new UsingIsUnderFunction();
        sw.start();
        runQueryFor(getEM(), 0L);
        System.out.println(String.format("%d %d %d IS_UNDER", Long.valueOf(numNodes), Long.valueOf(width), Long.valueOf(sw.elapsed(TimeUnit.MICROSECONDS))));
        sw.reset();

        System.setProperty("usePatch", "true");
        this.impl = new UsingPatch();
        sw.start();
        runQueryFor(getEM(), 0L);
        System.setProperty("usePatch", "false");
        System.out.println(String.format("%d %d %d DIRECT_SQL", Long.valueOf(numNodes), Long.valueOf(width), Long.valueOf(sw.elapsed(TimeUnit.MICROSECONDS))));
        sw.reset();

        this.impl = new UsingInternalLibrary();
        sw.start();
        runQueryFor(getEM(), 0L);
        System.out.println(String.format("%d %d %d DIRECT_SQL_INNER", Long.valueOf(numNodes), Long.valueOf(width), Long.valueOf(sw.elapsed(TimeUnit.MICROSECONDS))));
        sw.reset();

        this.impl = new UsingView();
        sw.start();
        runQueryFor(getEM(), 0L);
        System.out.println(String.format("%d %d %d VIEW", Long.valueOf(numNodes), Long.valueOf(width), Long.valueOf(sw.elapsed(TimeUnit.MICROSECONDS))));
        sw.reset();
    }

    private IsUnderClauseBuilder impl;

    @Override
    protected IsUnderClauseBuilder getImpl() {
        return impl;
    };
}
